@@ -28,10 +28,10 @@ class Authenticator:
     def __init__(
         self,
         email_address: str,
+        puid: str,
         password: str,
         proxy: str = None,
     ):
-        self.session_token = None
         self.email_address = email_address
         self.password = password
         self.proxy = proxy
@@ -43,6 +43,7 @@ class Authenticator:
         self.session.proxies.update(proxies)
         self.access_token: str = None
         self.user_agent = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/109.0.0.0 Safari/537.36"
+        self.session.cookies.set("_puid", puid)
 
     @staticmethod
     def url_encode(string: str) -> str:
@@ -55,16 +56,16 @@ class Authenticator:
 
     def begin(self) -> None:
         """
-        In part two, We make a request to https://explorer.api.openai.com/api/auth/csrf and grab a fresh csrf token
+        In part two, We make a request to https://chat.openai.com/api/auth/csrf and grab a fresh csrf token
         """
-        url = "https://explorer.api.openai.com/api/auth/csrf"
+        url = "https://chat.openai.com/api/auth/csrf"
         headers = {
-            "Host": "explorer.api.openai.com",
+            "Host": "chat.openai.com",
             "Accept": "*/*",
             "Connection": "keep-alive",
             "User-Agent": self.user_agent,
             "Accept-Language": "en-GB,en-US;q=0.9,en;q=0.8",
-            "Referer": "https://explorer.api.openai.com/auth/login",
+            "Referer": "https://chat.openai.com/auth/login",
             "Accept-Encoding": "gzip, deflate, br",
         }
         response = self.session.get(
@@ -73,54 +74,59 @@ class Authenticator:
         )
         if response.status_code == 200 and "json" in response.headers["Content-Type"]:
             csrf_token = response.json()["csrfToken"]
+            # self.session.cookies.set("__Host-next-auth.csrf-token", csrf_token)
             self.__part_one(token=csrf_token)
         else:
-            raise Error(
+            error = Error(
                 location="begin",
                 status_code=response.status_code,
                 details=response.text,
             )
+            print(error.details)
+            raise error
 
     def __part_one(self, token: str) -> None:
         """
         We reuse the token from part to make a request to /api/auth/signin/auth0?prompt=login
         """
-        url = "https://explorer.api.openai.com/api/auth/signin/auth0?prompt=login"
+        url = "https://chat.openai.com/api/auth/signin/auth0?prompt=login"
         payload = f"callbackUrl=%2F&csrfToken={token}&json=true"
         headers = {
-            "Host": "explorer.api.openai.com",
+            "Host": "chat.openai.com",
             "User-Agent": self.user_agent,
             "Content-Type": "application/x-www-form-urlencoded",
             "Accept": "*/*",
             "Sec-Gpc": "1",
             "Accept-Language": "en-US,en;q=0.8",
-            "Origin": "https://explorer.api.openai.com",
+            "Origin": "https://chat.openai.com",
             "Sec-Fetch-Site": "same-origin",
             "Sec-Fetch-Mode": "cors",
             "Sec-Fetch-Dest": "empty",
-            "Referer": "https://explorer.api.openai.com/auth/login",
+            "Referer": "https://chat.openai.com/auth/login",
             "Accept-Encoding": "gzip, deflate",
+            #
         }
         response = self.session.post(url=url, headers=headers, data=payload)
         if response.status_code == 200 and "json" in response.headers["Content-Type"]:
             url = response.json()["url"]
             if (
-                url
-                == "https://explorer.api.openai.com/api/auth/error?error=OAuthSignin"
+                url == "https://chat.openai.com/api/auth/error?error=OAuthSignin"
                 or "error" in url
             ):
-                raise Error(
+                error = Error(
                     location="__part_one",
                     status_code=response.status_code,
                     details="You have been rate limited. Please try again later.",
                 )
+                raise error
             self.__part_two(url=url)
         else:
-            raise Error(
+            error = Error(
                 location="__part_one",
                 status_code=response.status_code,
                 details=response.text,
             )
+            raise error
 
     def __part_two(self, url: str) -> None:
         """
@@ -134,7 +140,7 @@ class Authenticator:
             "Connection": "keep-alive",
             "User-Agent": self.user_agent,
             "Accept-Language": "en-US,en;q=0.9",
-            "Referer": "https://explorer.api.openai.com/",
+            "Referer": "https://chat.openai.com/",
         }
         response = self.session.get(
             url=url,
@@ -145,11 +151,12 @@ class Authenticator:
             state = state.split('"')[0]
             self.__part_three(state=state)
         else:
-            raise Error(
+            error = Error(
                 location="__part_two",
                 status_code=response.status_code,
                 details=response.text,
             )
+            raise error
 
     def __part_three(self, state: str) -> None:
         """
@@ -163,17 +170,18 @@ class Authenticator:
             "Connection": "keep-alive",
             "User-Agent": self.user_agent,
             "Accept-Language": "en-US,en;q=0.9",
-            "Referer": "https://explorer.api.openai.com/",
+            "Referer": "https://chat.openai.com/",
         }
         response = self.session.get(url, headers=headers)
         if response.status_code == 200:
             self.__part_four(state=state)
         else:
-            raise Error(
+            error = Error(
                 location="__part_three",
                 status_code=response.status_code,
                 details=response.text,
             )
+            raise error
 
     def __part_four(self, state: str) -> None:
         """
@@ -207,11 +215,12 @@ class Authenticator:
         if response.status_code == 302 or response.status_code == 200:
             self.__part_five(state=state)
         else:
-            raise Error(
+            error = Error(
                 location="__part_four",
                 status_code=response.status_code,
                 details="Your email address is invalid.",
             )
+            raise error
 
     def __part_five(self, state: str) -> None:
         """
@@ -239,19 +248,19 @@ class Authenticator:
             allow_redirects=False,
             data=payload,
         )
-        if response.status_code == 302 or response.status_code == 200:
-            new_state = re.findall(r"state=(.*)", response.text)[0]
-            new_state = new_state.split('"')[0]
-            self.__part_six(old_state=state, new_state=new_state)
+        if response.status_code == 302:
+            redirect_url = response.headers.get("location")
+            self.__part_six(old_state=state, redirect_url=redirect_url)
         else:
-            raise Error(
+            error = Error(
                 location="__part_five",
                 status_code=response.status_code,
                 details="Your credentials are invalid.",
             )
+            raise error
 
-    def __part_six(self, old_state: str, new_state) -> None:
-        url = f"https://auth0.openai.com/authorize/resume?state={new_state}"
+    def __part_six(self, old_state: str, redirect_url) -> None:
+        url = "https://auth0.openai.com" + redirect_url
         headers = {
             "Host": "auth0.openai.com",
             "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
@@ -260,26 +269,22 @@ class Authenticator:
             "Accept-Language": "en-GB,en-US;q=0.9,en;q=0.8",
             "Referer": f"https://auth0.openai.com/u/login/password?state={old_state}",
         }
-        response = self.session.get(
-            url,
-            headers=headers,
-            allow_redirects=False,
-        )
+        response = self.session.get(url, headers=headers, allow_redirects=False)
         if response.status_code == 302:
-            # Print redirect url
             redirect_url = response.headers.get("location")
             self.__part_seven(redirect_url=redirect_url, previous_url=url)
         else:
-            raise Error(
+            error = Error(
                 location="__part_six",
                 status_code=response.status_code,
                 details=response.text,
             )
+            raise error
 
     def __part_seven(self, redirect_url: str, previous_url: str) -> None:
         url = redirect_url
         headers = {
-            "Host": "explorer.api.openai.com",
+            "Host": "chat.openai.com",
             "Accept": "application/json",
             "Connection": "keep-alive",
             "User-Agent": self.user_agent,
@@ -287,39 +292,32 @@ class Authenticator:
             "Referer": previous_url,
         }
         response = self.session.get(
-            url,
-            headers=headers,
-            allow_redirects=False,
+            url, headers=headers, allow_redirects=True, cookies=self.session.cookies
         )
-        if response.status_code == 302:
-            self.session_token = response.cookies.get(
-                "__Secure-next-auth.session-token",
-            )
-            self.get_access_token()
+        if response.status_code == 200:
+            return
         else:
-            raise Error(
+            error = Error(
                 location="__part_seven",
                 status_code=response.status_code,
                 details=response.text,
             )
+            raise error
 
     def get_access_token(self):
         """
         Gets access token
         """
-        self.session.cookies.set(
-            "__Secure-next-auth.session-token",
-            self.session_token,
-        )
         response = self.session.get(
-            "https://explorer.api.openai.com/api/auth/session",
+            "https://chat.openai.com/api/auth/session",
         )
         if response.status_code == 200:
-            self.access_token = response.json()["accessToken"]
+            self.access_token = response.json().get("accessToken")
             return self.access_token
         else:
-            raise Error(
+            error = Error(
                 location="get_access_token",
                 status_code=response.status_code,
                 details=response.text,
             )
+            raise error
